@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/config.h 2.8 1997/12/27 12:07:51 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/config.h 2.19 1998/06/19 19:26:59 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0a.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2a.
   Configuration file management functions.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997 Andrew M. Bishop
+  This file Copyright 1997,98 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -16,9 +16,15 @@
 #ifndef CONFIG_H
 #define CONFIG_H    /*+ To stop multiple inclusions. +*/
 
+/* In config.c */
+
+int ReadConfigFile(int fd);
+
+
 /*+ The name of the configuration file. +*/
 extern char *ConfigFile;
 
+
 /* StartUp section */
 
 /*+ The port number to use for the HTTP proxy port. +*/
@@ -46,37 +52,52 @@ extern char *PassWord;
 extern int MaxServers;          /*+ in total. +*/
 extern int MaxFetchServers;     /*+ for fetching. +*/
 
+
 /* Options Section */
 
 /*+ The level of error logging (see ErrorLevel in errors.h) +*/
 extern int LogLevel,            /*+ in the config file for syslog and stderr. +*/
            DebugLevel;          /*+ on the command line for stderr. +*/
 
-/*+ The option to also fetch images. +*/
-extern int FetchImages;
-
-/*+ The option to also fetch frames. +*/
-extern int FetchFrames;
-
 /*+ The number of days to display in the index of the latest pages. +*/
 extern int IndexLatestDays;
 
 /*+ The option of a tag that can be added to the bottom of the spooled pages with the date and a refresh button. +*/
 extern int AddInfoRefresh;
 
-/*+ The option to always request the server for changes when it is the cache and online. +*/
+/*+ The maximum age of a cached page to use in preference while online. +*/
 extern int RequestChanged;
 
 /*+ The option to allow or ignore the 'Pragma: no-cache' request. +*/
 extern int PragmaNoCache;
 
-/* FTPOptions section */
+/*+ The option to not make requests while offline but to return an error. +*/
+extern int OfflineRequests;
 
-/*+ The anon-ftp username. +*/
-extern char *FTPUserName;
+/*+ The interval in days between monitoring the specified pages. +*/
+extern int MonitorInterval;
+
+/*+ The amount of time that a socket connection will wait for data. +*/
+extern int SocketTimeout;
+
+
+/* FetchOptions Section */
+
+/*+ The option to also fetch style sheets. +*/
+extern int FetchStyleSheets;
+
+/*+ The option to also fetch images. +*/
+extern int FetchImages;
+
+/*+ The option to also fetch frames. +*/
+extern int FetchFrames;
+
+/*+ The option to also fetch scripts. +*/
+extern int FetchScripts;
+
+/*+ The option to also fetch objects. +*/
+extern int FetchObjects;
 
-/*+ The anon-ftp password. +*/
-extern char *FTPPassWord;
 
 /* Purge section */
 
@@ -90,54 +111,67 @@ extern int DefaultPurgeAge;
 extern int PurgeCacheSize;
 
 
-/* In config.h */
-
-int ReadConfigFile(int fd);
-
 /* LocalHost section */
 
 char *GetLocalHost(int port);
-int IsLocalHost(char *host);
+int IsLocalHost(char *host,int port);
+
 
 /* LocalNet section */
 
 int IsLocalNetHost(char *host);
 
+
 /* AllowedConnect section */
 
 int IsAllowedConnect(char *host);
 
+
 /* DontCache section */
 
 int IsNotCached(char *proto,char *host,char *path);
 
+
 /* DontGet section */
 
 int IsNotGot(char *proto,char *host,char *path);
 
+
 /* DontGetRecursive section */
 
 int IsNotGotRecursive(char *proto,char *host,char *path);
 
+
 /* CensorHeader section */
 
-int IsCensoredHeader(char *line);
+char *CensoredHeader(char *line);
+
+
+/* FTPOptions section */
+
+int WhatFTPUserPass(char *host,char **user,char **pass);
+
 
 /* MIMETypes section */
 
 char *WhatMIMEType(char *path);
 
+
 /* Proxy section */
 
 char *WhichProxy(char *proto,char *host);
+char *WhatProxyAuth(char *proxy);
+
 
 /* Mirror section */
 
 int IsMirrored(char *proto,char *host,char **new_proto,char **new_host);
 char ***ListMirrors(char *proto);
 
+
 /* Purge section */
 
 int WhatPurgeAge(char *proto,char *host);
 
+
 #endif /* CONFIG_H */
