@@ -1,22 +1,23 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/errors.c 2.6 1997/12/29 17:15:47 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/errors.c 2.10.1.1 1998/07/09 18:26:16 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0a.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2c.
   Generate error messages in a standard format optionally to syslog and stderr.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97 Andrew M. Bishop
+  This file Copyright 1996,97,98 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
   ***************************************/
 
 
-/*+ Under Linux, FreeBSD, NetBSD, SGI, Ultrix, AIX and AT&T stdarg is used +*/
-#if defined(__linux__) || \
+/*+ Under Linux, FreeBSD, NetBSD, Solaris x86, SGI, Ultrix, AIX and AT&T stdarg is used +*/
+#if defined(__linux__) || defined(hpux) || \
     defined(__FreeBSD__) || defined(__NetBSD__) || \
-    defined(__sgi__) || defined(__ultrix__) || \
+    (defined(__i386__) && defined(__sun__) && defined(__svr4__)) || \
+    defined(__sgi__) || defined(__sgi) || defined(__ultrix__) || \
     defined(_AIX) || defined(_ATT4)
 #define USE_STD_ARG
 #endif
@@ -37,12 +38,17 @@
 
 #include <syslog.h>
 
+#if defined (hpux)
+/* HP/UX does not declare these in <syslog.h> */
+extern int syslog (int pri, const char *message, ...);
+extern int openlog (const char *ident, int logopt, int facility);
+extern int closelog (void);
+#endif
+
 #include <errno.h>
 
 #if defined(__sun__) && !defined(__svr4__)
-
 /* SunOS 4.x does not have strerror(). */
-
 char* strerror(int err);
 
 extern int sys_nerr;
@@ -55,11 +61,15 @@ char* strerror(int err)
  else
     return("Unknown error");
 }
-
 #endif /* defined(__sun__) && !defined(__svr4__) */
 
 #include <netdb.h>
 
+#if defined (hpux)
+/* HP/UX has h_errno but does not decalre it in any header file */
+extern int h_errno;
+#endif
+
 /* A function to get an error message for h_errno. */
 
 char* str_h_error(int err);
@@ -104,9 +114,24 @@ char* str_h_error(int err)
  return("Unknown error");
 }
 
-#include "config.h"
 #include "errors.h"
 
+/* Allow this file to be compiled without needing to use the configuration
+   file reading functions in config.c.
+   - Requires some replacement variables.  */
+
+#ifdef NO_CONFIG
+
+int UseSyslog=0;
+int LogLevel=Inform;
+int DebugLevel=-1;
+
+#else
+
+#include "config.h"
+
+#endif /* NO_CONFIG */
+
 
 /*+ The name of the program. +*/
 static char *program=NULL;
@@ -118,10 +143,10 @@ static pid_t pid;
 static time_t last_time;
 
 /*+ The error messages. +*/
-static char *ErrorString[]={"Debug:" ,"Information:","Important:","Warning:" ,"Fatal:"};
+static char *ErrorString[]={"ExtraDebug:","Debug:" ,"Information:","Important:","Warning:" ,"Fatal:"};
 
 /*+ The priority to apply to syslog messages. +*/
-static int ErrorPriority[]={LOG_DEBUG,LOG_INFO      ,LOG_NOTICE  ,LOG_WARNING,LOG_ERR};
+static int ErrorPriority[]={-1           ,LOG_DEBUG,LOG_INFO      ,LOG_NOTICE  ,LOG_WARNING,LOG_ERR};
 
 /*+ The error facility to use, +*/
 static int use_syslog=0,        /*+ use syslog. +*/
@@ -185,13 +210,22 @@ char *PrintMessage(ErrorLevel errlev,const char* fmt, ...)
  int i,j;
  time_t this_time=time(NULL);
 
+ /* Shortcut (bypass if debug) */
+
+ if(errlev<=Debug && (errlev<LogLevel && errlev<DebugLevel))
+    return(NULL);
+
+ /* Periodic timestamp */
+
  if((this_time-last_time)>3600)
    {
     last_time=this_time;
     if(use_stderr)
-       fprintf(stderr,"%s[%d] Timestamp: %s",program,pid,ctime(&this_time));
+       fprintf(stderr,"%s[%ld] Timestamp: %s",program,(long)pid,ctime(&this_time));
    }
 
+ /* Parsing of printf style arguments. */
+
  if(string)
     free(string);
 
@@ -260,11 +294,13 @@ char *PrintMessage(ErrorLevel errlev,const char* fmt, ...)
 
  va_end(ap);
 
+ /* Output the result. */
+
  if(UseSyslog && use_syslog && errlev>=LogLevel && ErrorPriority[errlev]!=-1)
     syslog(ErrorPriority[errlev],"%s",string);
 
  if(use_stderr && ((DebugLevel==-1 && errlev>=LogLevel) || (DebugLevel!=-1 && errlev>=DebugLevel)))
-    fprintf(stderr,"%s[%d] %s %s",program,pid,ErrorString[errlev],string);
+    fprintf(stderr,"%s[%ld] %s %s",program,(long)pid,ErrorString[errlev],string);
 
  if(errlev==Fatal)
     exit(2);
