@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/ftp.c 1.12 1998/01/10 16:16:30 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/ftp.c 1.23 1998/06/10 20:47:29 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0c.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2.
   Functions for getting URLs using FTP.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -33,8 +33,8 @@
 /*+ Set this to 1 to see the full dialog with the FTP server. +*/
 #define DEBUG_FTP 0
 
-/*+ Set to true if we are using a proxy. +*/
-static int proxied=0;
+/*+ Set to the name of the proxy if there is one. +*/
+static char *proxy=NULL;
 
 /*+ The file descriptor of the socket +*/
 static int server_ctrl=-1,      /*+ for the control connection to the server. +*/
@@ -53,10 +53,6 @@ static int nbufferhead=0,       /*+ in total for the head . +*/
            nbuffertail=0,       /*+ in total for the tail . +*/
            nreadtail=0;         /*+ that have been read from the tail. +*/
 
-/*+ The part we are on head, body or tail. +*/
-static int part=0;
-
-
 static char *htmlise_dir_entry(char *line);
 
 
@@ -72,15 +68,16 @@ char *FTP_Open(URL *Url)
 {
  char *msg=NULL;
  char *colon;
- char *proxy=WhichProxy(Url->proto,Url->host);
  char *server_host=NULL;
  int server_port=Protocols[Protocol_FTP].defport;
 
  /* Sort out the host. */
 
- proxied=proxy && !IsLocalNetHost(Url->host);
+ proxy=WhichProxy(Url->proto,Url->host);
+ if(IsLocalNetHost(Url->host))
+    proxy=NULL;
 
- if(proxied)
+ if(proxy)
    {
     server_host=(char*)malloc(strlen(proxy)+1);
     strcpy(server_host,proxy);
@@ -108,13 +105,6 @@ char *FTP_Open(URL *Url)
 
  free(server_host);
 
- /* Take a simple route if it is proxied. */
-
- if(proxied)
-    return(msg);
-
- /* exit */
-
  return(msg);
 }
 
@@ -126,29 +116,42 @@ char *FTP_Open(URL *Url)
 
   URL *Url The URL to get.
 
-  char *request The HTTP request for the URL.
+  char *request_head The head of the HTTP request for the URL.
+
+  char *request_body The body of the HTTP request for the URL.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *FTP_Request(URL *Url,char *request)
+char *FTP_Request(URL *Url,char *request_head,char *request_body)
 {
  char *msg=NULL,*str=NULL;
  char *path,*file=NULL;
  char *host,*mimetype;
  char *msg_reply=NULL;
  int i,l,port_l,port_h;
+ char *user,*pass;
 
  /* Take a simple route if it is proxied. */
 
- if(proxied)
+ if(proxy)
    {
-    if(write_string(server_ctrl,request)==-1)
+    char *new_request_head=MakeRequestAuthorised(proxy,request_head);
+
+    PrintMessage(ExtraDebug,"Outgoing Request Head (to proxy)\n%s",new_request_head);
+
+    if(write_string(server_ctrl,new_request_head)==-1)
        msg=PrintMessage(Warning,"Failed to write to remote FTP proxy [%!s].");
+    if(request_body && write_string(server_ctrl,request_body)==-1)
+       msg=PrintMessage(Warning,"Failed to write to remote FTP proxy [%!s].");
+
+    if(request_head!=new_request_head)
+       free(new_request_head);
+
     return(msg);
    }
 
  /* Else Sort out the path. */
 
- path=UrlDecode(Url->path,0);
+ path=URLDecode(Url->path,0);
  if(path[strlen(path)-1]=='/')
    {
     path[strlen(path)-1]=0;
@@ -163,8 +166,12 @@ char *FTP_Request(URL *Url,char *request)
           break;
          }
 
+ WhatFTPUserPass(Url->host,&user,&pass);
+
  /* send all the RFC959 commands. */
 
+ server_data=-1;
+
  do
    {
     str=read_line_or_timeout(server_ctrl,str);
@@ -203,7 +210,7 @@ char *FTP_Request(URL *Url,char *request)
 
  /* Login */
 
- if(write_formatted(server_ctrl,"USER %s\r\n",FTPUserName)==-1)
+ if(write_formatted(server_ctrl,"USER %s\r\n",user)==-1)
    {
     msg=PrintMessage(Warning,"Failed to write 'USER' command to remote FTP host [%!s].");
     return(msg);
@@ -213,7 +220,7 @@ char *FTP_Request(URL *Url,char *request)
    {
     str=read_line_or_timeout(server_ctrl,str);
 #if DEBUG_FTP
-    PrintMessage(Debug,"FTP: sent 'USER %s'; got: %s",FTPUserName,str);
+    PrintMessage(Debug,"FTP: sent 'USER %s'; got: %s",user,str);
 #endif
    }
  while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -233,7 +240,7 @@ char *FTP_Request(URL *Url,char *request)
 
  if(atoi(str)==331)
    {
-    if(write_formatted(server_ctrl,"PASS %s\r\n",FTPPassWord)==-1)
+    if(write_formatted(server_ctrl,"PASS %s\r\n",pass)==-1)
       {
        msg=PrintMessage(Warning,"Failed to write 'PASS' command to remote FTP host [%!s].");
        return(msg);
@@ -243,7 +250,7 @@ char *FTP_Request(URL *Url,char *request)
       {
        str=read_line_or_timeout(server_ctrl,str);
 #if DEBUG_FTP
-       PrintMessage(Debug,"FTP: sent 'PASS %s'; got: %s",FTPPassWord,str);
+       PrintMessage(Debug,"FTP: sent 'PASS %s'; got: %s",pass,str);
 #endif
 
        if(!file && !*path && str && isdigit(str[0]) && isdigit(str[1]) && isdigit(str[2]) && str[3]=='-')
@@ -339,7 +346,7 @@ char *FTP_Request(URL *Url,char *request)
       }
     while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
 
-    if(!str || (atoi(str)!=250 && atoi(str)!=550))
+    if(!str || (atoi(str)!=250 && atoi(str)!=550 && atoi(str)!=501))
       {
        if(str)
          {
@@ -354,27 +361,20 @@ char *FTP_Request(URL *Url,char *request)
 
     if(atoi(str)==250)
       {
-       char *msg=
-       "HTTP/1.0 302 Is a directory\r\n"
-       "Content-Type: text/html\r\n"
-       "Location: %s/\r\n"
-       "\r\n"
-       "<HTML>\n"
-       "<HEAD>\n"
-       "<TITLE>FTP Directory requires trailing '/'</TITLE>\n"
-       "</HEAD>\n"
-       "<BODY>\n"
-       "<h1>Index of %s</h1>\n"
-       "You need to specify a URL with a '/' at the end for a directory\n"
-       "<br>\n"
-       "<a href=\"%s/\">%s/</a>\n";
+       char *loc=(char*)malloc(strlen(Url->link)+2);
 
-       bufferhead=(char*)malloc(4*strlen(Url->name)+strlen(msg)+1);
-       sprintf(bufferhead,msg,Url->name,Url->name,Url->name,Url->name);
+       strcpy(loc,Url->link);
+       strcat(loc,"/");
 
-       buffer=NULL;
+       bufferhead=HTMLMessageHead(-1,302,"Is a directory",
+                                  "Location",loc,
+                                  NULL);
 
-       buffertail="</BODY>\n</HTML>\n";
+       buffer=HTMLMessageBody(-1,"FTPDirRedirect",
+                              "url",Url->link,
+                              NULL);
+
+       free(loc);
 
        goto near_end;
       }
@@ -462,7 +462,7 @@ char *FTP_Request(URL *Url,char *request)
  server_data=OpenClientSocket(host,port_l+256*port_h);
  init_buffer(server_data);
 
- if(!server_data)
+ if(server_data==-1)
    {
     msg=PrintMessage(Warning,"Cannot open the FTP data connection [%!s].");
     return(msg);
@@ -510,7 +510,12 @@ char *FTP_Request(URL *Url,char *request)
    }
 
  if(str && (atoi(str)==150 || atoi(str)==125))
-    *str='2',*(str+1)='0',*(str+2)='0';
+   {
+    char *p=str;
+    while(*p!='\r' && *p!='\n')
+       p++;
+    *p=0;
+   }
  else
    {
     if(str)
@@ -526,27 +531,14 @@ char *FTP_Request(URL *Url,char *request)
 
  /* Prepare the HTTP header. */
 
- bufferhead=(char*)malloc(strlen(str)+strlen(mimetype)+32);
- strcpy(bufferhead,"HTTP/1.0 ");
- strcat(bufferhead,str);
- strcat(bufferhead,"Content-Type: ");
- strcat(bufferhead,mimetype);
- strcat(bufferhead,"\r\n");
- strcat(bufferhead,"\r\n");
-
+ bufferhead=HTMLMessageHead(-1,200,str+4,
+                            "Content-Type",mimetype,
+                            NULL);
  buffer=NULL;
  buffertail=NULL;
 
  if(!file)
    {
-    char *head="<HTML>\n"
-               "<HEAD>\n"
-               "<TITLE>Index of %s</TITLE>\n"
-               "</HEAD>\n"
-               "<BODY>\n"
-               "<h1>Index of %s</h1>\n";
-    char *path=UrlDecode(Url->name,0);
-
     if(msg_reply)
       {
        char *old_msg_reply=msg_reply;
@@ -554,23 +546,13 @@ char *FTP_Request(URL *Url,char *request)
        free(old_msg_reply);
       }
 
-    bufferhead=(char*)realloc((void*)bufferhead,strlen(bufferhead)+(msg_reply?strlen(msg_reply):0)+strlen(head)+2*strlen(path)+16);
-    sprintf(&bufferhead[strlen(bufferhead)],head,path,path);
+    buffer=HTMLMessageBody(-1,"FTPDir-Head",
+                           "url",URLDecode(Url->name,0),
+                           "message",msg_reply?msg_reply:"",
+                           NULL);
 
-    if(msg_reply)
-      {
-       strcat(bufferhead,"<pre>\n");
-       strcat(bufferhead,msg_reply);
-       strcat(bufferhead,"</pre>\n");
-      }
-
-    strcat(bufferhead,"<pre>\n");
-
-    buffertail="</pre>\n"
-               "</BODY>\n"
-               "</HTML>\n";
-
-    free(path);
+    buffertail=HTMLMessageBody(-1,"FTPDir-Tail",
+                               NULL);
    }
 
 near_end:
@@ -578,10 +560,8 @@ near_end:
  if(msg_reply)
     free(msg_reply);
 
- part=1;
-
- nbuffer=nread=0;
  nbufferhead=strlen(bufferhead); nreadhead=0;
+ nbuffer=buffer?strlen(buffer):0; nread=0;
  nbuffertail=buffertail?strlen(buffertail):0; nreadtail=0;
 
  free(str);
@@ -592,163 +572,100 @@ near_end:
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Read bytes from the source of information for the URL.
+  Read a line from the header of the reply for the URL.
 
-  int FTP_Read Returns the number of bytes read on success, -1 on error.
+  char *FTP_ReadHead Returns the next line of data, NULL on EOF.
+
+  char *line The previous line read.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char *FTP_ReadHead(char *line)
+{
+ char *l=line;
+ int m;
+
+ /* Take a simple route if it is proxied. */
+
+ if(proxy)
+    return(read_line_or_timeout(server_ctrl,line));
+ 
+ /* Else send the header. */
+
+ if(nreadhead==nbufferhead)
+    return(NULL);
+
+ for(m=nreadhead;m<nbufferhead;m++)
+    if(bufferhead[m]=='\n')
+       break;
+
+ m++;
+ l=(char*)realloc((void*)l,m-nreadhead+1);
+ strncpy(l,&bufferhead[nreadhead],m-nreadhead);
+ l[m-nreadhead]=0;
+
+ nreadhead=m;
+
+ return(l);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Read bytes from the body of the reply for the URL.
+
+  int FTP_ReadBody Returns the number of bytes read on success, -1 on error.
 
   char *s A string to fill in with the information.
 
   int n The number of bytes to read.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int FTP_Read(char *s,int n)
+int FTP_ReadBody(char *s,int n)
 {
  int m=0;
 
  /* Take a simple route if it is proxied. */
 
- if(proxied)
+ if(proxy)
     return(read_data_or_timeout(server_ctrl,s,n));
  
- /* Else send the header then the data then the tail. */
+ /* Else send the data then the tail. */
 
- if(part==1)
+ if(server_data==-1)            /* Redirection */
    {
-    for(;nreadhead<nbufferhead && m<n;nreadhead++,m++)
-       s[m]=bufferhead[nreadhead];
+    for(;nread<nbuffer && m<n;nread++,m++)
+       s[m]=buffer[nread];
 
-    if(nreadhead==nbufferhead)
-       part=2;
+    for(;nreadtail<nbuffertail && m<n;nreadtail++,m++)
+       s[m]=buffertail[nreadtail];
    }
- else if(part==2 && server_data==-1)
-    part=3;
- else if(part==2)
+ else if(!buffer && !buffertail) /* File not dir entry. */
+    m=read_data_or_timeout(server_data,s,n);
+ else if(buffer && buffertail)  /* Middle of dir entry */
    {
-    if(buffertail && !buffer)
+    for(;nread<nbuffer && m<n;nread++,m++)
+       s[m]=buffer[nread];
+
+    if(nread==nbuffer)
       {
        buffer=htmlise_dir_entry(buffer);
-
-       if(!buffer)
-          part=3;
-       else
-         {
-          nbuffer=strlen(buffer);
-          nread=0;
-         }
+       if(buffer)
+          nbuffer=strlen(buffer),nread=0;
       }
 
-    if(buffer)
-      {
-       for(;nread<nbuffer && m<n;nread++,m++)
-          s[m]=buffer[nread];
-
-       if(nread==nbuffer)
-         {free(buffer);buffer=NULL;}
-      }
-    else
-      {
-       m=read_data_or_timeout(server_data,s,n);
-
-       if(m==0)
-          part=3;
-      }
-   }
-
- if(part==3)
-   {
-    if(nreadtail==nbuffertail)
-      m=0;
-    else
+    if(!buffer)
        for(;nreadtail<nbuffertail && m<n;nreadtail++,m++)
           s[m]=buffertail[nreadtail];
    }
+ else if(!buffer && buffertail) /* End of dir entry. */
+   {
+    for(;nreadtail<nbuffertail && m<n;nreadtail++,m++)
+       s[m]=buffertail[nreadtail];
+   }
 
  return(m);
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Read a line from the source of information for the URL.
-
-  char *FTP_ReadLine Returns the next line of data, NULL on EOF.
-
-  char *line The previous line read.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-char *FTP_ReadLine(char *line)
-{
- char *l=line;
-
- /* Take a simple route if it is proxied. */
-
- if(proxied)
-    return(read_line_or_timeout(server_ctrl,line));
- 
- /* Else send the header then the data. */
-
- if(part==1)
-   {
-    int m;
-
-    for(m=nreadhead;m<nbufferhead;m++)
-       if(bufferhead[m]=='\n')
-          break;
-
-    m++;
-    l=(char*)realloc((void*)l,m-nreadhead+1);
-    strncpy(l,&bufferhead[nreadhead],m-nreadhead);
-    l[m-nreadhead]=0;
-
-    nreadhead=m;
-
-    if(nreadhead==nbufferhead)
-       part=2;
-   }
- else if(part==2 && server_data==-1)
-    part=3;
- else if(part==2)
-   {
-    if(buffertail && !buffer)
-       l=htmlise_dir_entry(line);
-    else if(buffertail && buffer)
-       {
-        l=(char*)realloc((void*)l,nbuffer-nread+1);
-        strcpy(l,&buffer[nread]);
-        free(buffer);
-        buffer=NULL;
-       }
-    else
-       l=read_line_or_timeout(server_data,line);
-
-    if(!l)
-       part=3;
-   }
-
- if(part==3)
-   {
-    int m;
-
-    if(nreadtail==nbuffertail)
-      {if(l)free(l);l=NULL;}
-    else
-      {
-       for(m=nreadtail;m<nbuffertail;m++)
-          if(buffertail[m]=='\n')
-             break;
-
-       m++;
-       l=(char*)realloc((void*)l,m-nreadtail+1);
-       strncpy(l,&buffertail[nreadhead],m-nreadtail);
-       l[m-nreadtail]=0;
-
-       nreadtail=m;
-      }
-   }
-
- return(l);
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Close a connection opened using FTP.
 
@@ -762,7 +679,7 @@ int FTP_Close(void)
 
  /* Take a simple route if it is proxied. */
 
- if(proxied)
+ if(proxy)
     return(CloseSocket(server_ctrl));
 
  /* Else say goodbye and close all of the sockets, */
@@ -788,6 +705,12 @@ int FTP_Close(void)
  if(bufferhead)
     free(bufferhead);
 
+ if(buffer)
+    free(buffer);
+
+ if(buffertail)
+    free(buffertail);
+
  return(err);
 }
 
@@ -852,7 +775,7 @@ static char *htmlise_dir_entry(char *line)
     endlc=*endl;
 
     *endl=0;
-    urlenc=UrlEncode(link);
+    urlenc=URLEncode(link);
     *endl=endlc;
 
     ll=(char*)malloc(strlen(l)+strlen(urlenc)+24);
