@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 2.10 1998/01/03 17:57:45 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 2.19 1998/06/21 09:08:54 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0c.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2a.
   A user level program to interact with the server.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -21,6 +21,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include "version.h"
 #include "wwwoffle.h"
 #include "misc.h"
 #include "config.h"
@@ -36,13 +37,15 @@ typedef enum _Action
 {
  None,                          /*+ Undecided. +*/
  Online,                        /*+ Tell the server that we are online. +*/
+ Autodial,                      /*+ Tell the server that we are in autodial mode. +*/
  Offline,                       /*+ Tell the server that we are offline. +*/
  Fetch,                         /*+ Tell the server to fetch the requested pages. +*/
  Config,                        /*+ Tell the server to re-read the configuration file. +*/
  Purge,                         /*+ Tell the server to purge pages. +*/
  Kill,                          /*+ Tell the server to exit. +*/
  Get,                           /*+ Tell the server to get pages. +*/
- Output                         /*+ Get a page from the server and output it. +*/
+ Output,                        /*+ Get a page from the server and output it. +*/
+ OutputWithHeader               /*+ Get a page and the headers from the server and output it. +*/
 }
 Action;
 
@@ -62,10 +65,12 @@ int n_url_file_list=0;
 int main(int argc, char** argv)
 {
  int i;
- int force=0,images=0,frames=0,recursive_depth=0,recursive_mode=0;
+ int recursive_depth=0,recursive_mode=0,force=0;
+ int stylesheets=0,images=0,frames=0,scripts=0,objects=0;
 
  Action action=None;
 
+ char *env=NULL;
  char *host=NULL;
  int port=0;
 
@@ -90,6 +95,15 @@ int main(int argc, char** argv)
        continue;
       }
 
+    if(!strcmp(argv[i],"-O"))
+      {
+       if(action!=None)
+         {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
+       action=OutputWithHeader;
+       argv[i]=NULL;
+       continue;
+      }
+
     if(!strcmp(argv[i],"-F"))
       {
        if(action!=None && action!=Get)
@@ -102,8 +116,52 @@ int main(int argc, char** argv)
        continue;
       }
 
+    if(!strncmp(argv[i],"-g",2))
+      {
+       if(action!=None && action!=Get)
+         {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
+       action=Get;
+       if(strchr(argv[i]+2,'S'))
+         {
+          if(stylesheets)
+            {fprintf(stderr,"wwwoffle: Only one '-gS' argument may be given.\n"); exit(1);}
+          stylesheets=1;
+         }
+       if(strchr(argv[i]+2,'i'))
+         {
+          if(images)
+            {fprintf(stderr,"wwwoffle: Only one '-gi' argument may be given.\n"); exit(1);}
+          images=1;
+         }
+       if(strchr(argv[i]+2,'f'))
+         {
+          if(frames)
+            {fprintf(stderr,"wwwoffle: Only one '-gf' argument may be given.\n"); exit(1);}
+          frames=1;
+         }
+       if(strchr(argv[i]+2,'s'))
+         {
+          if(scripts)
+            {fprintf(stderr,"wwwoffle: Only one '-gs' argument may be given.\n"); exit(1);}
+          scripts=1;
+         }
+       if(strchr(argv[i]+2,'o'))
+         {
+          if(objects)
+            {fprintf(stderr,"wwwoffle: Only one '-go' argument may be given.\n"); exit(1);}
+          objects=1;
+         }
+       if(argv[i][2]==0)
+          fprintf(stderr,"wwwoffle: The '-g' option does nothing on its own.\n");
+
+       argv[i]=NULL;
+       continue;
+      }
+
     if(!strcmp(argv[i],"-i"))
       {
+       fprintf(stderr,"wwwoffle: The '-i' option is being replaced by '-gi'.\n");
+
        if(action!=None && action!=Get)
          {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
        action=Get;
@@ -116,6 +174,8 @@ int main(int argc, char** argv)
 
     if(!strcmp(argv[i],"-f"))
       {
+       fprintf(stderr,"wwwoffle: The '-f' option is being replaced by '-gf'.\n");
+
        if(action!=None && action!=Get)
          {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
        action=Get;
@@ -169,7 +229,7 @@ int main(int argc, char** argv)
 
           port=atoi(colon);
 
-          if(port<0 || port>=65536)
+          if(port<=0 || port>=65536)
             {fprintf(stderr,"wwwoffle: The port number %d '%s' is invalid.\n",port,argv[i]); exit(1);}
          }
 
@@ -204,6 +264,15 @@ int main(int argc, char** argv)
        continue;
       }
 
+    if(!strcmp(argv[i],"-auto") || !strcmp(argv[i],"-autodial"))
+      {
+       if(action!=None)
+         {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
+       action=Autodial;
+       argv[i]=NULL;
+       continue;
+      }
+
     if(!strcmp(argv[i],"-off") || !strcmp(argv[i],"-offline"))
       {
        if(action!=None)
@@ -263,6 +332,37 @@ int main(int argc, char** argv)
 
  /* Initialise things. */
 
+ if(!ConfigFile && !host && (env=getenv("WWWOFFLE_PROXY")))
+   {
+    if(*env=='/')
+       ConfigFile=env;
+    else
+      {
+       char *colon1,*colon2;
+
+       host=(char*)malloc(strlen(env)+1);
+       strcpy(host,env);
+
+       if((colon1=strchr(host,':')))
+         {
+          *colon1=0;
+          if((colon2=strchr(colon1+1,':')))
+            {
+             *colon2=0;
+             if(action==Get || action==Output || action==OutputWithHeader)
+                port=atoi(colon1+1);
+             else
+                port=atoi(colon2+1);
+            }
+          else
+             port=atoi(colon1+1);
+
+          if(port<=0 || port>=65536)
+            {fprintf(stderr,"wwwoffle: The port number %d '%s' is invalid.\n",port,env); exit(1);}
+         }
+      }
+   }
+
  InitErrorHandler("wwwoffle",0,1);
 
  if(ConfigFile)
@@ -275,7 +375,7 @@ int main(int argc, char** argv)
 
  /* The connections to the WWWOFFLE server. */
 
- if(action!=Get && action!=Output)
+ if(action!=Get && action!=Output && action!=OutputWithHeader)
    {
     int socket;
     char *buffer=NULL;
@@ -293,6 +393,8 @@ int main(int argc, char** argv)
 
     if(action==Online)
        write_string(socket,"WWWOFFLE ONLINE\r\n");
+    else if(action==Autodial)
+       write_string(socket,"WWWOFFLE AUTODIAL\r\n");
     else if(action==Offline)
        write_string(socket,"WWWOFFLE OFFLINE\r\n");
     else if(action==Fetch)
@@ -321,33 +423,16 @@ int main(int argc, char** argv)
    {
     URL *Url;
     struct stat buf;
-    char refresh[32];
+    char *refresh;
 
-    strcpy(refresh,"refresh");
-    if(images)
-       strcat(refresh,"-images");
-    if(frames)
-       strcat(refresh,"-frames");
-    if(force)
-       strcat(refresh,"-force");
-    if(recursive_depth)
-      {
-       if(recursive_mode==1)
-          strcat(refresh,"-dir");
-       else if(recursive_mode==2)
-          strcat(refresh,"-host");
-       else /* recursive_mode==3 */
-          strcat(refresh,"-any");
-       sprintf(&refresh[strlen(refresh)],"-%d",recursive_depth);
-      }
-    if(!images && !frames && !recursive_depth && !force)
-       strcat(refresh,"-none");
+    refresh=CreateRefreshPath(recursive_depth,recursive_mode,force,
+                              stylesheets,images,frames,scripts,objects);
 
     for(i=0;i<n_url_file_list;i++)
        if(strcmp(url_file_list[i],"-") && stat(url_file_list[i],&buf))
          {
           int socket;
-          char *buffer=NULL;
+          char buffer256[257];
 
           socket=OpenClientSocket(host?host:"localhost",port?port:HTTP_Port);
           init_buffer(socket);
@@ -359,16 +444,14 @@ int main(int argc, char** argv)
 
           printf("Getting: %s\n",Url->name);
 
-          write_formatted(socket,"GET /%s/%s/%s HTTP/1.0\r\n"
+          write_formatted(socket,"GET %s/?%s HTTP/1.0\r\n"
+                                 "Pragma: wwwoffle\r\n"
                                  "Accept: */*\r\n"
                                  "\r\n",
-                          refresh,Url->proto,Url->hostp);
+                          refresh,Url->name);
 
-          while((buffer=read_line(socket,buffer)))
-            {
-             fputs(buffer,stdout);
-             fflush(stdout);
-            }
+          while(read_data(socket,buffer256,256)>0)
+             ;
 
           CloseSocket(socket);
 
@@ -455,7 +538,7 @@ int main(int argc, char** argv)
        else
           PrintMessage(Warning,"The file '%s' is not a regular file.",url_file_list[i]);
    }
- else /* action==Output */
+ else /* action==Output or action==OutputWithHeader */
    {
     URL *Url;
     int socket;
@@ -473,30 +556,40 @@ int main(int argc, char** argv)
 
     Url=SplitURL(url_file_list[0]);
 
-    fprintf(stderr,"Getting: %s\n",Url->name);
+    if(action!=Output && action!=OutputWithHeader)
+       fprintf(stderr,"Getting: %s\n",Url->name);
 
     write_formatted(socket,"GET %s HTTP/1.0\r\n"
+                           "Pragma: wwwoffle\r\n"
                            "Accept: */*\r\n"
                            "\r\n",
                     Url->name);
 
     if((buffer=read_line(socket,buffer)))
       {
-       char *willget="HTTP/1.0 404 WWWOFFLE Will Get\r\n"; /* This line must not be changed (see messages.c). */
+       char *willget="HTTP/1.0 404 WWWOFFLE Will Get\r\n";
        int status;
 
        sscanf(buffer,"%*s %d",&status);
 
        if(!strcmp(willget,buffer))
           fprintf(stderr,"The URL is not in the cache but has been requested.\n");
-       else if(status>=300 && status<400)
+       else if((status>=300 && status<400) && action!=OutputWithHeader)
           fprintf(stderr,"The URL has been moved, check with a browser.\n");
-       else if(status!=200)
+       else if(status!=200 && action!=OutputWithHeader)
           fprintf(stderr,"The URL returns an error message, check with a browser.\n");
        else
          {
-          while((buffer=read_line(socket,buffer)) && buffer[0]!='\r' && buffer[0]!='\n')
-             ;
+          if(action==OutputWithHeader)
+             fputs(buffer,stdout);
+
+          while((buffer=read_line(socket,buffer)))
+            {
+             if(action==OutputWithHeader)
+                fputs(buffer,stdout);
+             if(buffer[0]=='\r' || buffer[0]=='\n')
+                break;
+            }
 
           while((nbytes=read_data(socket,buffer256,256))>0)
              fwrite(buffer256,1,nbytes,stdout);
@@ -524,32 +617,41 @@ int main(int argc, char** argv)
 
 static void usage(int verbose)
 {
- fputs("\n"
-       "WWWOFFLE - World Wide Web Offline Explorer - Version 2.0\n"
-       "\n",stderr);
+ fprintf(stderr,
+         "\n"
+         "WWWOFFLE - World Wide Web Offline Explorer - Version %s\n"
+         "\n",WWWOFFLE_VERSION);
 
  if(verbose)
-    fputs("(c) Andrew M. Bishop 1996,97 [       amb@gedanken.demon.co.uk ]\n"
-          "                             [http://www.gedanken.demon.co.uk/]\n"
-          "\n",stderr);
+    fprintf(stderr,
+            "(c) Andrew M. Bishop 1996,97,98 [       amb@gedanken.demon.co.uk ]\n"
+            "                                [http://www.gedanken.demon.co.uk/]\n"
+            "\n");
 
- fputs("Usage: wwwoffle -h\n"
-       "       wwwoffle -online | -offline | -fetch | -config | -purge | -kill\n"
-       "       wwwoffle -o <url>\n"
-       "       wwwoffle [-i] [-f] [-F] [-(d|r|R)[<depth>]] <url> ...\n"
-       "       wwwoffle [-i] [-f] [-F] [-(d|r|R)[<depth>]] [<file>|-] ...\n"
-       "\n"
-       "Any of these can also take:  [-p <host>[:<port>] | -c <config-file>]\n"
-       "\n",stderr);
+ fprintf(stderr,
+         "Usage: wwwoffle -h\n"
+         "       wwwoffle -online | -autodial | -offline | -fetch\n"
+         "       wwwoffle -config | -purge | -kill\n"
+         "       wwwoffle [-o|-O] <url>\n"
+         "       wwwoffle [-g[Sisfo]] [-F] [-(d|r|R)[<depth>]] <url> ...\n"
+         "       wwwoffle [-g[Sisfo]] [-F] [-(d|r|R)[<depth>]] [<file>|-] ...\n"
+         "\n"
+         "Any of these can also take:  [-p <host>[:<port>] | -c <config-file>]\n"
+         "The environment variable WWWOFFLE_PROXY can be set instead of -p or -c options.\n"
+         "\n");
 
  if(verbose)
     fprintf(stderr,
             "wwwoffle -h          : Display this help.\n"
             "\n"
-            "wwwoffle -online     : Indicate to the server that the network is active.\n"
+            "wwwoffle -on[line]   : Indicate to the server that the network is active.\n"
             "                       (Proxy requests will be fetched from remote hosts.)\n"
             "\n"
-            "wwwoffle -offline    : Indicate to the server that the network is inactive.\n"
+            "wwwoffle -auto[dial] : Indicate to the server that the network is automatic.\n"
+            "                       (Proxy requests will be fetched from remote hosts\n"
+            "                        ONLY if they are not already cached.)\n"
+            "\n"
+            "wwwoffle -off[line]  : Indicate to the server that the network is inactive.\n"
             "                       (Proxy requests will be fetched from cache or recorded.)\n"
             "\n"
             "wwwoffle -fetch      : Force the server to fetch the pages that are recorded.\n"
@@ -565,9 +667,10 @@ static void usage(int verbose)
             "wwwoffle <file> ...  : Fetch the URLs that are links in the specified file.\n"
             "\n"
             " -o                  : Fetch the URL and output it on the standard output.\n"
+            " -O                  : As above but include the HTTP header.\n"
             "\n"
-            " -i                  : Fetch the images included in the specified URLs.\n"
-            " -f                  : Fetch the frames included in the specified URLs.\n"
+            " -g[Sisfo]           : Fetch the items included in the specified URLs.\n"
+            "                       (S=stylesheets, i=images, f=frames, s=scripts, o=objects)\n"
             " -F                  : Force the url to be refreshed even if already cached.\n"
             " -(d|r|R)[<depth>]   : Fetch pages linked to the URLs and their links,\n"
             "                       going no more than <depth> steps (default 1).\n"
@@ -577,10 +680,16 @@ static void usage(int verbose)
             "\n"
             " -p <host>[:<port>]  : The host name and port number to talk to the demon on.\n"
             "                       (Defaults to localhost for the server and\n"
-            "                        %d for server port, %d for http proxy port).\n"
+            "                        %d for control port, %d for http proxy port).\n"
             "\n"
             " -c <config-file>    : The name of the configuration file with the hostname,\n"
             "                       port number and the password (if any).\n"
+            "\n"
+            "WWWOFFLE_PROXY       : An environment variable that can be set to either the\n"
+            "                       name of the config file (absolute path) or the hostname\n"
+            "                       and port number (both proxy and control) for the proxy.\n"
+            "                       e.g. \"/var/spool/wwwoffle/wwwoffle.conf\",\n"
+            "                       \"localhost:8080:8081\" or \"localhost:8080\" are valid.\n"
             "\n",DEF_WWWOFFLE_PORT,DEF_HTTP_PORT);
 
  if(verbose)
