@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 2.19 1998/01/03 17:59:47 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 2.45 1998/06/22 18:14:57 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0c.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2b.
   A header file for all of the programs wwwoffle, wwwoffled.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -32,9 +32,6 @@
 /*+ The default configuration file directory. +*/
 #define DEF_CONF "/var/spool/wwwoffle"
 
-/*+ The default age of pages in days before purging. +*/
-#define DEF_PURGE_AGE 14
-
 /*+ The absolute maximum number of servers to fork in total. +*/
 #define MAX_SERVERS 64
 /*+ The default total number of servers to fork. +*/
@@ -45,8 +42,8 @@
 /*+ The default number of servers to fork for fetching pages. +*/
 #define DEF_MAX_FETCH_SERVERS 4
 
-/*+ The time in seconds to wait while reading from a socket. +*/
-#define SOCKET_TIMEOUT 120
+/*+ The number of pretime directories to create (lasttime history). +*/
+#define NUM_PREVTIME_DIR 3
 
 /* The values in the section above can be changed if required, see INSTALL.
    ------------------------------------------------------------ */
@@ -67,6 +64,7 @@ void PurgeCache(int fd);
 int OpenOutgoingSpoolFile(int rw);
 void CloseOutgoingSpoolFile(int fd,URL *Url);
 int ExistsOutgoingSpoolFile(URL *Url);
+char *HashOutgoingSpoolFile(URL *Url);
 char *DeleteOutgoingSpoolFile(URL *Url);
 
 int OpenWebpageSpoolFile(int rw,URL *Url);
@@ -74,53 +72,79 @@ char *DeleteWebpageSpoolFile(URL *Url,int all);
 void TouchWebpageSpoolFile(URL *Url);
 int ExistsWebpageSpoolFile(URL *Url);
 
+void CreateBackupWebpageSpoolFile(URL *Url,int fd);
+void RestoreBackupWebpageSpoolFile(URL *Url);
+void DeleteBackupWebpageSpoolFile(URL *Url);
+
+char *DeleteLastTimeSpoolFile(URL *Url);
+int CreateLastTimeSpoolFile(URL *Url);
+
+int CreateMonitorSpoolFile(URL *Url,int interval);
+char *DeleteMonitorSpoolFile(URL *Url);
+
 char *FileNameToURL(char *file);
 char *URLToFileName(URL *Url);
 
 /* In parse.c */
 
-char *ParseRequest(int req,char **url);
-int RequestChanges(int fd,char **request);
-char *MovedLocation(URL *Url,char *reply);
+char *ParseRequest(int fd,char **request_head,char **request_body);
+
+int RequestChanges(int fd,char **request_head);
+char *MovedLocation(URL *Url,char *reply_head);
 char *RequestURL(char *url,char *referer);
-char *ModifyRequest(URL *Url,char *request);
-char *MakeRequestNonProxy(char *request);
-char *ParseReply(URL *Url,int *status);
 
-/* In messages.c */
+char *ModifyRequest(URL *Url,char *request_head);
 
-void ServerError(int fd,char *string);
-void RemoteHostError(int fd,char *url,char *reason);
-void CantRefreshPosted(int fd,char *url);
-void WillGetURL(int fd,char *url,int already);
-void RefreshRedirect(int fd,char *url);
-void WelcomePage(int fd);
-void IllegalPage(int fd,char *pathargs);
-void IllegalProto(int fd,char *url,char *proto);
-void NotModified(int fd);
+char *MakeRequestAuthorised(char *proxy,char *request_head);
+char *MakeRequestNonProxy(char *request_head);
+
+int ParseReply(URL *Url,char **reply_head);
+
+char *GetHTTPHeader(char *head,char* line);
+
+/* In messages.c (messages.l) */
+
+void LocalPage(int fd,char *path,char *request_head);
+char *HTMLMessage(int fd,int status_val,char *status_str,char *location,char *template, ...);
+char *HTMLMessageHead(int fd,int status_val,char *status_str, ...);
+char *HTMLMessageBody(int fd,char *template, ...);
 
 /* In index.c */
 
-void IndexPage(int fd,char *path,char *args);
+void IndexPage(int fd,URL *Url);
 
 /* In control.c */
 
-void ControlPage(int fd,char *path,char *args,char *request);
+void ControlPage(int fd,URL *Url,char *request_head,char *request_body);
 
 /* In configedit.c */
 
-void ConfigEditPage(int fd,char *args,char *request);
+void ConfigEditPage(int fd,char *args,char *request_body);
 
 /* In refresh.c */
 
-char *RefreshPage(int fd,char *path,char *args,char *url,char *request,int is_online);
+char *RefreshPage(int fd,URL *Url,char *request_body,int *recurse);
+void ParseRecurseOptions(char *method);
+int RecurseFetch(URL *Url);
+char *CreateRefreshPath(int recursive_depth,int recursive_mode,int force,
+                        int stylesheets,int images,int frames,int scripts,int objects);
+
+/* In monitor.c */
+
+void MonitorPage(int fd,URL *Url,char *request_body);
+void RequestMonitoredPages(void);
+void MonitorTimes(char *name,int *interval,int *last,int *next);
 
 /* In html.c (html.l) */
 
 int ParseHTML(int fd,URL *Url,int html);
+char *MetaRefresh(void);
+char **ListStyleSheets(void);
 char **ListImages(void);
-char **ListLinks(void);
 char **ListFrames(void);
+char **ListScripts(void);
+char **ListObjects(void);
+char **ListLinks(void);
 int GetHTMLEnd(void);
 
 /* In wwwoffles.c */
