@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 2.6 1997/12/27 12:07:52 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 2.10 1998/04/30 17:39:18 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0a.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2.
   A demon program to maintain the database and spawn the servers.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97 Andrew M. Bishop
+  This file Copyright 1996,97,98 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -15,6 +15,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 
 #include <sys/wait.h>
@@ -26,12 +27,18 @@
 #include <errno.h>
 #include <signal.h>
 
+#include "version.h"
 #include "wwwoffle.h"
 #include "misc.h"
 #include "config.h"
 #include "sockets.h"
 #include "errors.h"
 
+#if defined (hpux)
+/* HP/UX has no `seteuid'. Instead it uses `setresuid' to set `uid', `euid', and `suid' simultaneously. */
+#define setegid(egid)   setresgid (-1, (egid), -1)
+#define seteuid(euid)   setresuid (-1, (euid), -1)
+#endif
 
 static void usage(int verbose);
 static void demoninit(void);
@@ -45,7 +52,7 @@ static void sighup(int signum);
 int http_fd=-1,                 /*+ for the HTTP connections. +*/
     wwwoffle_fd=-1;             /*+ for the WWWOFFLE connections. +*/
 
-/*+ The online / offline status. +*/
+/*+ The online / offline /autodial status. +*/
 int online=0;
 
 /*+ The current number active servers +*/
@@ -125,7 +132,7 @@ int main(int argc, char** argv)
  if(ReadConfigFile(2))
     PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigFile);
 
- PrintMessage(Important,"WWWOFFLE Demon Version 2.0 started");
+ PrintMessage(Important,"WWWOFFLE Demon Version %s started",WWWOFFLE_VERSION);
  PrintMessage(Inform,"WWWOFFLE Read Configuration File.");
 
  if(WWWOFFLE_Gid != -1)
@@ -209,17 +216,17 @@ int main(int argc, char** argv)
       {
        if(FD_ISSET(wwwoffle_fd,&readfd))
          {
-          char *host;
+          char *host,*ip;
           int port,client;
 
           client=AcceptConnect(wwwoffle_fd);
           init_buffer(client);
 
-          if(client>=0 && !SocketRemoteName(client,&host,NULL,&port))
+          if(client>=0 && !SocketRemoteName(client,&host,&ip,&port))
             {
-             if(IsAllowedConnect(host))
+             if(IsAllowedConnect(host) || IsAllowedConnect(ip))
                {
-                PrintMessage(Important,"WWWOFFLE Connection from %s.",host);
+                PrintMessage(Important,"WWWOFFLE Connection from %s (%s).",host,ip);
 
                 CommandConnect(client);
 
@@ -236,17 +243,17 @@ int main(int argc, char** argv)
 
        if(FD_ISSET(http_fd,&readfd))
          {
-          char *host;
+          char *host,*ip;
           int port,client;
 
           client=AcceptConnect(http_fd);
           init_buffer(client);
 
-          if(client>=0 && !SocketRemoteName(client,&host,NULL,&port))
+          if(client>=0 && !SocketRemoteName(client,&host,&ip,&port))
             {
-             if(IsAllowedConnect(host))
+             if(IsAllowedConnect(host) || IsAllowedConnect(ip))
                {
-                PrintMessage(Inform,"HTTP Proxy connection from %s.",host);
+                PrintMessage(Inform,"HTTP Proxy connection from %s (%s).",host,ip);
                 ForkServer(client,1);
                }
              else
@@ -307,10 +314,10 @@ int main(int argc, char** argv)
          if(exitval==3)
             fetching=0;
 
-         if(exitval==4 && online)
+         if(exitval==4 && online==1)
             fetching=1;
 
-         if(!online)
+         if(online!=1)
             fetching=0;
         }
 
@@ -378,17 +385,20 @@ int main(int argc, char** argv)
 
 static void usage(int verbose)
 {
- fputs("\n"
-       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 2.0a\n"
-       "\n",stderr);
+ fprintf(stderr,
+         "\n"
+         "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version %s\n"
+         "\n",WWWOFFLE_VERSION);
 
  if(verbose)
-    fputs("(c) Andrew M. Bishop 1996,97 [       amb@gedanken.demon.co.uk ]\n"
-          "                             [http://www.gedanken.demon.co.uk/]\n"
-          "\n",stderr);
+    fprintf(stderr,
+            "(c) Andrew M. Bishop 1996,97,98 [       amb@gedanken.demon.co.uk ]\n"
+            "                                [http://www.gedanken.demon.co.uk/]\n"
+            "\n");
 
- fputs("Usage: wwwoffled [-h] [-c <config-file>] [-d [<log-level>]]\n"
-       "\n",stderr);
+ fprintf(stderr,
+         "Usage: wwwoffled [-h] [-c <config-file>] [-d [<log-level>]]\n"
+         "\n");
 
  if(verbose)
     fprintf(stderr,
@@ -398,8 +408,8 @@ static void usage(int verbose)
             "                         (Default %s).\n"
             "\n"
             "   -d [<log-level>]    : Do not detach from the terminal and use stderr.\n"
-            "                         The log-level is a number from 0 to 5.\n"
-            "\n",ConfigFile);
+            "                         0 <= log-level <= %d (default in config file).\n"
+            "\n",ConfigFile,Fatal+1);
 
  exit(0);
 }
