@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffles.c 2.29 1998/01/08 19:14:26 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffles.c 2.67.1.1 1998/07/10 15:00:39 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.0c.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2c.
   A server to fetch the required pages.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -42,11 +42,12 @@ typedef enum _Mode
  Real,                          /*+ From server host to cache and client. +*/
  RealNoCache,                   /*+ From server host to client. +*/
  RealRefresh,                   /*+ Refresh the page, forced from index. +*/
+ SpoolOrReal,                   /*+ Spool if already cached else Real. +*/
  Fetch,                         /*+ From server host to cache. +*/
  Spool,                         /*+ From cache to client. +*/
  SpoolGet,                      /*+ Not in cache so record request in outgoing. +*/
  SpoolWillGet,                  /*+ Not in cache but is in outgoing. +*/
- SpoolRefresh,                  /*+ Refresh the page, forced from index. +*/
+ SpoolRefresh,                  /*+ Refresh the page, forced from refresh page. +*/
  SpoolPragma                    /*+ Refresh the page, forced from browser by 'Pragma: no-cache'. +*/
 }
 Mode;
@@ -67,15 +68,15 @@ Mode;
 int wwwoffles(int online,int browser,int client)
 {
  int outgoing=-1,spool=-1;
-
- char *request=NULL,*reply=NULL,*url=NULL;
-
- char *mirrorProto=NULL,*mirrorHost=NULL;
-
- URL *Url;
-
- Mode mode=None;
  int fetch_again=0;
+ char *mirrorProto=NULL,*mirrorHost=NULL;
+ char *request_head,*request_body;
+ char *reply_head,*reply_body;
+ int reply_status=-1;
+ char *url;
+ URL *Url;
+ Mode mode=None;
+ int lasttime_exists=0,spool_exists=0;
 
  /* Initialise things. */
 
@@ -83,14 +84,22 @@ int wwwoffles(int online,int browser,int client)
 
  InitErrorHandler("wwwoffles",-1,-1); /* change name nothing else */
 
- if(online && browser)
+ if(online==1 && browser)
     mode=Real;
- else if(online && !browser)
+ else if(online==1 && !browser)
     mode=Fetch;
+ else if(online==-1 && browser)
+    mode=SpoolOrReal;
  else if(!online && browser)
     mode=Spool;
  else
-    PrintMessage(Fatal,"Started in a mode that is not allowed.");
+   {
+    PrintMessage(Fatal,"Started in a mode that is not allowed (online=%d, browser=%d).",online,browser);
+    if(client!=-1)
+       HTMLMessage(client,500,"WWWOFFLE Server Error",NULL,"ServerError",
+                   "error","Started server in illegal mode.",
+                   NULL);
+   }
 
  /* Set up the client file. */
 
@@ -110,16 +119,18 @@ int wwwoffles(int online,int browser,int client)
 
  /* Get the URL from the request. */
 
- if(mode==Real || mode==Spool)
-    request=ParseRequest(client,&url);
+ if(mode==Real || mode==Spool || mode==SpoolOrReal)
+    url=ParseRequest(client,&request_head,&request_body);
  else /* mode==Fetch */
-    request=ParseRequest(outgoing,&url);
+    url=ParseRequest(outgoing,&request_head,&request_body);
 
  if(!url)
    {
+    PrintMessage(Warning,"Could not parse HTTP request (%s).",request_head?"Parse error":"Empty request");
     if(mode!=Fetch)
-       ServerError(client,request?"Cannot parse the HTTP request":"The HTTP request was empty");
-    PrintMessage(Warning,"Could not parse HTTP request (%s).",request?"Parse error":"Empty request");
+       HTMLMessage(client,500,"WWWOFFLE Server Error",NULL,"ServerError",
+                   "error",request_head?"Cannot parse the HTTP request":"The HTTP request was empty",
+                   NULL);
     exit(1);
    }
 
@@ -129,38 +140,57 @@ int wwwoffles(int online,int browser,int client)
 
  PrintMessage(Debug,"proto='%s'; host='%s'; path='%s'; args='%s'.",Url->proto,Url->host,Url->path,Url->args);
 
+ PrintMessage(ExtraDebug,"Incoming Request Head (from browser)\n%s",request_head);
+
+ if(Url->user)
+   {
+    PrintMessage(Inform,"URLs with username and password are not allowed.",Url->proto);
+    if(mode==Real || mode==Spool || mode==SpoolOrReal)
+       HTMLMessage(client,404,"WWWOFFLE Password Not Allowed",NULL,"PasswordNotAllowed",
+                   "url",Url->name,
+                   NULL);
+    exit(1);
+   }
+
+ /* Initial sanity checks. */
+
+ if(IsNotGot(Url->proto,Url->host,Url->path))
+   {
+    PrintMessage(Inform,"The server '%s://%s' and/or path '%s' is on the list not to get.",Url->proto,Url->host,Url->path);
+    if(mode==Real || mode==Spool || mode==SpoolOrReal)
+       HTMLMessage(client,404,"WWWOFFLE Host Not Got",NULL,"HostNotGot",
+                   "url",Url->name,
+                   NULL);
+    exit(0);
+   }
+
  if(!Url->Protocol)
    {
-    if(mode==Real || mode==Spool)
-       IllegalProto(client,Url->name,Url->proto);
     PrintMessage(Inform,"The protocol '%s' is not available.",Url->proto);
+    if(mode==Real || mode==Spool || mode==SpoolOrReal)
+       HTMLMessage(client,404,"WWWOFFLE Illegal Protocol",NULL,"IllegalProtocol",
+                   "url",Url->name,
+                   "protocol",Url->proto,
+                   NULL);
     exit(1);
    }
 
  /* Change the mode based on the URL as required. */
- /* (At this point mode is Spool, Real or Fetch only.) */
+ /* (At this point mode is Spool, Real, SpoolOrReal or Fetch only.) */
 
- if(IsNotGot(Url->proto,Url->host,Url->path))
-   {
-    char* colon=strchr(Url->host,':');
-    if(colon) *colon=0;
-    if(mode==Real || mode==Spool)
-       RemoteHostError(client,Url->name,"is on the list of hosts and paths not to get.");
-    PrintMessage(Inform,"The server '%s://%s' and/or path '%s' is on the list not to get.",Url->proto,Url->host,Url->path);
-    exit(0);
-   }
+ /* A local URL. */
 
  if(Url->local)
    {
-    int newurl=0;
-
     if(!strncmp("/refresh",Url->path,8))
       {
-       url=RefreshPage(client,Url->path,Url->args,Url->name,request,online);
+       int recurse=0;
 
-       if(url==(char*)1)
-          newurl=-1;
-       else if(url)
+       url=RefreshPage(client,Url,request_body,&recurse);
+
+       if(!url)
+          exit(0);
+       else
          {
           FreeURL(Url);
           Url=SplitURL(url);
@@ -169,37 +199,70 @@ int wwwoffles(int online,int browser,int client)
             {
              PrintMessage(Inform,"It is not possible to refresh a URL that was posted.");
              if(mode!=Fetch)
-                CantRefreshPosted(client,Url->name);
+                HTMLMessage(client,404,"WWWOFFLE Cant Refresh Posted",NULL,"CantRefreshPosted",
+                            "url",Url->name,
+                            NULL);
              exit(0);
             }
-          else if(!*Url->host || IsLocalNetHost(Url->host))
-             mode=RealNoCache;
-          else if(mode==Real)
+
+          if(recurse)
             {
-             mode=RealRefresh;
-             DeleteWebpageSpoolFile(Url,0);
+             if(mode==Spool || mode==Real || mode==SpoolOrReal)
+               {
+                mode=SpoolGet;
+                free(request_head);
+                request_head=RequestURL(Url->name,NULL);
+                if(request_body)
+                   free(request_body);
+                request_body=NULL;
+               }
+             fetch_again=1;
             }
-          else if(mode==Spool)
+          else
             {
-             if(ExistsWebpageSpoolFile(Url))
+             if(mode==Spool)
                 mode=SpoolRefresh;
-             else
-                if(ExistsOutgoingSpoolFile(Url))
-                   mode=SpoolWillGet;
-                else
-                   mode=SpoolGet;
+             else if(mode==SpoolOrReal || mode==Real)
+               {
+                DeleteWebpageSpoolFile(Url,0);
+                mode=RealRefresh;
+               }
             }
-          newurl=1;
+
+          PrintMessage(Inform,"new URL='%s'.",url);
+          PrintMessage(Debug,"new proto='%s'; host='%s'; path='%s'; args='%s'.",Url->proto,Url->host,Url->path,Url->args);
          }
       }
     else if(mode==Fetch)
+      {
        PrintMessage(Inform,"The request to fetch a page from the local host is ignored.");
-    else if(!strcmp(Url->path,"/") && !Url->args)
-       WelcomePage(client);
+       exit(0);
+      }
+    else if(!strchr(Url->path+1,'/') && !Url->args)
+      {
+       LocalPage(client,Url->path,request_head);
+       exit(0);
+      }
     else if(!strncmp("/index/",Url->path,7))
-       IndexPage(client,&Url->path[7],Url->args);
+      {
+       IndexPage(client,Url);
+       exit(0);
+      }
     else if(!strncmp("/control/",Url->path,9))
-       ControlPage(client,&Url->path[9],Url->args,request);
+      {
+       ControlPage(client,Url,request_head,request_body);
+       exit(0);
+      }
+    else if(!strncmp("/monitor",Url->path,8))
+      {
+       MonitorPage(client,Url,request_body);
+       exit(0);
+      }
+    else if(!strncmp("/local/",Url->path,7) && !Url->args)
+      {
+       LocalPage(client,Url->path,request_head);
+       exit(0);
+      }
     else
       {
        int i;
@@ -212,33 +275,48 @@ int wwwoffles(int online,int browser,int client)
              Url->pathp[strlen(Protocols[i].name)+1]=0;
              sprintf(url,"%s://%s",Url->pathp+1,&Url->pathp[strlen(Protocols[i].name)+2]);
              Url->pathp[strlen(Protocols[i].name)+1]='/';
-             newurl=1;
              break;
             }
 
-       if(!newurl)
-          IllegalPage(client,Url->pathp);
+       if(i==NProtocols)
+         {
+          HTMLMessage(client,404,"WWWOFFLE Page Not Found",NULL,"PageNotFound",
+                      "url",Url->name,
+                      NULL);
+          exit(0);
+         }
 
        FreeURL(Url);
        Url=SplitURL(url);
       }
 
-    if(!newurl)
-       exit(0);
-    else if(newurl==-1)
-       exit(4);
+    /* Repeat initial sanity checks. */
 
-    PrintMessage(Inform,"new URL='%s'.",url);
-    PrintMessage(Debug,"new proto='%s'; host='%s'; path='%s'; args='%s'.",Url->proto,Url->host,Url->path,Url->args);
+    if(IsNotGot(Url->proto,Url->host,Url->path))
+      {
+       PrintMessage(Inform,"The server '%s://%s' and/or path '%s' is on the list not to get.",Url->proto,Url->host,Url->path);
+       if(mode==Real || mode==Spool || mode==SpoolOrReal)
+          HTMLMessage(client,404,"WWWOFFLE Host Not Got",NULL,"HostNotGot",
+                      "url",Url->name,
+                      NULL);
+       exit(0);
+      }
 
     if(!Url->Protocol)
       {
-       if(mode==Real || mode==Spool)
-          IllegalProto(client,Url->name,Url->proto);
        PrintMessage(Inform,"The protocol '%s' is not available.",Url->proto);
+       if(mode==Real || mode==Spool || mode==SpoolOrReal)
+          HTMLMessage(client,404,"WWWOFFLE Illegal Protocol",NULL,"IllegalProtocol",
+                      "url",Url->name,
+                      "protocol",Url->proto,
+                      NULL);
        exit(1);
       }
    }
+ else if(mode==Fetch)
+    ParseRecurseOptions(NULL);
+
+ /* Check for a mirror site. */
 
  if(IsMirrored(Url->proto,Url->host,&mirrorProto,&mirrorHost))
    {
@@ -256,36 +334,45 @@ int wwwoffles(int online,int browser,int client)
 
     if(!Url->Protocol)
       {
-       if(mode==Real || mode==Spool)
-          IllegalProto(client,Url->name,Url->proto);
        PrintMessage(Inform,"The protocol '%s' is not available.",Url->proto);
+       if(mode==Real || mode==Spool || mode==SpoolOrReal)
+          HTMLMessage(client,404,"WWWOFFLE Illegal Protocol",NULL,"IllegalProtocol",
+                      "url",Url->name,
+                      "protocol",Url->proto,
+                      NULL);
        exit(1);
       }
    }
 
+ /* Check if it needs to be cached. */
+
  if(IsLocalNetHost(Url->host))
    {
-    if(mode==Real || mode==Spool)
+    if(mode==Real || mode==Spool || mode==SpoolOrReal)
        mode=RealNoCache;
+    else if(mode==SpoolWillGet || mode==SpoolGet)
+       ;
     else
       {
        PrintMessage(Inform,"The request to fetch a page from the local network host '%s' is ignored.",Url->host);
        exit(0);
       }
    }
- else if(mode==Real && IsNotCached(Url->proto,Url->host,Url->path))
+ else if((mode==Real || mode==SpoolOrReal) && IsNotCached(Url->proto,Url->host,Url->path))
    {
     mode=RealNoCache;
    }
 
- if(!strncasecmp(request,"POST",4))
+ /* Check if it was posted. */
+
+ if(!strncasecmp(request_head,"POST",4))
    {
     if(mode==Spool)
        mode=SpoolGet;
    }
  else if(Url->args && *Url->args=='!')
    {
-    if(mode==Real)
+    if(mode==Real || mode==SpoolOrReal)
        mode=Spool;
     else if(mode==Fetch)
       {
@@ -294,12 +381,14 @@ int wwwoffles(int online,int browser,int client)
       }
    }
 
- if(strstr(request,"\nIf-Modified-Since:"))
+ /* Check if it is a conditional request. */
+
+ if(mode!=RealNoCache && GetHTTPHeader(request_head,"If-Modified-Since:"))
    {
-    char *bol=strstr(request,"\nIf-Modified-Since:")+1,*eol=strchr(bol,'\n');
+    char *bol=GetHTTPHeader(request_head,"If-Modified-Since:"),*eol=strchr(bol,'\n');
     *eol++=0;
 
-    if(mode==Spool)
+    if(mode==Spool || mode==SpoolOrReal)
       {
        spool=OpenWebpageSpoolFile(1,Url);
 
@@ -315,7 +404,11 @@ int wwwoffles(int online,int browser,int client)
           close(spool);
 
           if(since>buf.st_mtime)
-            {NotModified(client);exit(0);}
+            {
+             HTMLMessageHead(client,304,"WWWOFFLE Not Modified",
+                             NULL);
+             exit(0);
+            }
          }
       }
 
@@ -324,11 +417,15 @@ int wwwoffles(int online,int browser,int client)
     *bol=0;
    }
 
- if(PragmaNoCache && strstr(request,"\nPragma: no-cache"))
+ /* Check if a refresh is needed based on pragma, request changes, autodial. */
+
+ spool_exists=ExistsWebpageSpoolFile(Url);
+
+ if(PragmaNoCache && GetHTTPHeader(request_head,"Pragma: no-cache"))
    {
     if(mode==Spool || mode==SpoolGet)
       {
-       if(ExistsWebpageSpoolFile(Url))
+       if(spool_exists)
           mode=SpoolPragma;
        else
           if(ExistsOutgoingSpoolFile(Url))
@@ -336,39 +433,54 @@ int wwwoffles(int online,int browser,int client)
           else
              mode=SpoolGet;
       }
+    else if(mode==SpoolOrReal)
+       mode=Real;
+
     /* (mode==Fetch || mode==Real) are left unchanged, not modified as below. */
    }
- else if(mode==Spool && !ExistsWebpageSpoolFile(Url))
+ else if(mode==Spool && !spool_exists)
    {
     if(ExistsOutgoingSpoolFile(Url))
        mode=SpoolWillGet;
     else
        mode=SpoolGet;
    }
- else if(mode==Fetch && ExistsWebpageSpoolFile(Url))
+ else if(mode==Fetch && spool_exists && !fetch_again)
    {
     spool=OpenWebpageSpoolFile(1,Url);
     init_buffer(spool);
 
-    if(RequestChanges(spool,&request)==1)
+    if(RequestChanges(spool,&request_head)==1)
        TouchWebpageSpoolFile(Url);
     else
        exit(0);
 
     close(spool);
    }
- else if(mode==Real && ExistsWebpageSpoolFile(Url))
+ else if(mode==Real && spool_exists)
    {
     spool=OpenWebpageSpoolFile(1,Url);
     init_buffer(spool);
 
-    if(RequestChanged && RequestChanges(spool,&request)==1)
+    if(RequestChanged>=0 && RequestChanges(spool,&request_head)==1)
        TouchWebpageSpoolFile(Url);
     else
        mode=Spool;
 
     close(spool);
    }
+ else if(mode==SpoolOrReal)
+   {
+    if(spool_exists)
+       mode=Spool;
+    else
+       mode=Real;
+   }
+
+ /* Create the last time file. */
+
+ if(mode==Real || mode==Fetch)
+    lasttime_exists=CreateLastTimeSpoolFile(Url);
 
  /* Set up the file descriptor for the spool file. */
 
@@ -379,11 +491,16 @@ int wwwoffles(int online,int browser,int client)
 
     if(spool==-1)
       {
-       if(mode==Real)
-          ServerError(client,"Cannot open the spooled web page to write.");
        PrintMessage(Warning,"Cannot open the spooled web page to write.");
+       if(mode==Real)
+          HTMLMessage(client,500,"WWWOFFLE Server Error",NULL,"ServerError",
+                      "error","Cannot open the spooled web page to write.",
+                      NULL);
        exit(1);
       }
+
+    if(spool_exists)
+       CreateBackupWebpageSpoolFile(Url,spool);
    }
  else if(mode==Spool || mode==SpoolPragma)
    {
@@ -392,22 +509,26 @@ int wwwoffles(int online,int browser,int client)
 
     if(spool==-1)
       {
-       ServerError(client,"Cannot open the spooled web page to read.");
        PrintMessage(Warning,"Cannot open the spooled web page to read.");
+       HTMLMessage(client,500,"WWWOFFLE Server Error",NULL,"ServerError",
+                   "error","Cannot open the spooled web page to read.",
+                   NULL);
        exit(1);
       }
    }
 
  /* Set up the outgoing file. */
 
- if(mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma)
+ if((mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma) && OfflineRequests)
    {
     outgoing=OpenOutgoingSpoolFile(0);
 
     if(outgoing==-1)
       {
-       ServerError(client,"Cannot open the outgoing request to write.");
        PrintMessage(Warning,"Cannot open the outgoing request to write.");
+       HTMLMessage(client,500,"WWWOFFLE Server Error",NULL,"ServerError",
+                   "error","Cannot open the outgoing request to write.",
+                   NULL);
        exit(1);
       }
    }
@@ -424,10 +545,20 @@ int wwwoffles(int online,int browser,int client)
          {
           lseek(spool,0,SEEK_SET);
           ftruncate(spool,0);
-          RemoteHostError(spool,Url->name,err);
+          HTMLMessage(spool,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                      "url",Url->name,
+                      "reason",err,
+                      "cache","yes",
+                      "backup",spool_exists?"yes":"",
+                      NULL);
          }
        if(mode==Real || mode==RealNoCache)
-          RemoteHostError(client,Url->name,err);
+          HTMLMessage(client,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                      "url",Url->name,
+                      "reason",err,
+                      "cache","",
+                      "backup",spool_exists?"yes":"",
+                      NULL);
        exit(1);
       }
    }
@@ -435,13 +566,13 @@ int wwwoffles(int online,int browser,int client)
  /* Modify the header (Censor / Cannonicalise URL / POST / HTTP-1.1 etc). */
 
  if(mode==Real || mode==RealNoCache || mode==Fetch || mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma)
-    request=ModifyRequest(Url,request);
+    request_head=ModifyRequest(Url,request_head);
 
  /* Write request to remote server or outgoing file. */
 
  if(mode==Real || mode==RealNoCache || mode==Fetch)
    {
-    char *err=(Url->Protocol->request)(Url,request);
+    char *err=(Url->Protocol->request)(Url,request_head,request_body);
 
     if(err)
       {
@@ -449,54 +580,82 @@ int wwwoffles(int online,int browser,int client)
          {
           lseek(spool,0,SEEK_SET);
           ftruncate(spool,0);
-          RemoteHostError(spool,Url->name,err);
+          HTMLMessage(spool,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                      "url",Url->name,
+                      "reason",err,
+                      "cache","yes",
+                      "backup",spool_exists?"yes":"",
+                      NULL);
          }
        if(mode==Real || mode==RealNoCache)
-          RemoteHostError(client,Url->name,err);
+          HTMLMessage(client,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                      "url",Url->name,
+                      "reason",err,
+                      "cache","",
+                      "backup",spool_exists?"yes":"",
+                      NULL);
+       else if(mode==Fetch && client!=-1)
+          write_formatted(client,"Fetching %s [Server Connection Error]\n",Url->name);
        exit(1);
       }
-    else if(mode==Fetch && client!=-1)
-       write_formatted(client,"Fetching %s\n",Url->name);
    }
- else if(mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma)
+ else if((mode==SpoolGet || mode==SpoolRefresh || mode==SpoolPragma) && OfflineRequests)
    {
-    int err=write_string(outgoing,request);
+    int err=write_string(outgoing,request_head);
 
     if(err==-1)
       {
-       ServerError(client,"Cannot write the outgoing request.");
        PrintMessage(Warning,"Cannot write the outgoing request.");
+       HTMLMessage(client,500,"WWWOFFLE Server Error",NULL,"ServerError",
+                   "error","Cannot write the outgoing request.",
+                   NULL);
        exit(1);
       }
+
+    if(request_body)
+       write_string(outgoing,request_body);
    }
 
  /* Parse the reply */
 
  if(mode==Real || mode==RealNoCache || mode==Fetch)
    {
-    int status=0;
+    reply_status=ParseReply(Url,&reply_head);
 
-    reply=ParseReply(Url,&status);
-
-    if(!reply)
+    if(!reply_head)
       {
+       PrintMessage(Warning,"Timed out reading the reply.");
        if(mode==Real || mode==Fetch)
          {
           lseek(spool,0,SEEK_SET);
           ftruncate(spool,0);
-          RemoteHostError(spool,Url->name,"the server timed out before sending the reply.");
+          HTMLMessage(spool,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                      "url",Url->name,
+                      "reason","TimeoutReply",
+                      "cache","yes",
+                      "backup",spool_exists?"yes":"",
+                      NULL);
          }
        if(mode==Real || mode==RealNoCache)
-          RemoteHostError(client,Url->name,"the server timed out before sending the reply.");
-       PrintMessage(Warning,"Timed out reading the reply.");
+          HTMLMessage(client,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                      "url",Url->name,
+                      "reason","TimeoutReply",
+                      "cache","",
+                      "backup",spool_exists?"yes":"",
+                      NULL);
+       else if(mode==Fetch && client!=-1)
+          write_formatted(client,"Fetching %s [Server Reply Error]\n",Url->name);
        exit(1);
       }
 
-    if(mode==Fetch && (status==301 || status==302))
-      {
-       char *new_url=MovedLocation(Url,reply);
+    PrintMessage(ExtraDebug,"Incoming Reply Head (from server/proxy)\n%s",reply_head);
 
-       PrintMessage(Inform,"This URL has been moved to %s.",new_url);
+    if(mode==Fetch && (reply_status==301 || reply_status==302))
+      {
+       char *new_url=MovedLocation(Url,reply_head);
+
+       if(client!=-1)
+          write_formatted(client,"Fetching %s [Page Moved]\n",Url->name);
 
        if(!new_url)
           PrintMessage(Warning,"Cannot parse the reply for the new location.");
@@ -505,6 +664,8 @@ int wwwoffles(int online,int browser,int client)
           char *new_request=RequestURL(new_url,Url->name);
           int new_outgoing=OpenOutgoingSpoolFile(0);
 
+          PrintMessage(Inform,"This URL has been moved to %s.",new_url);
+
           if(new_outgoing==-1)
              PrintMessage(Warning,"Cannot open the new outgoing request to write.");
           else
@@ -520,20 +681,34 @@ int wwwoffles(int online,int browser,int client)
           free(new_url);
          }
       }
-    else if(status==304)
+    else if(reply_status==304)
       {
        PrintMessage(Inform,"Server page is not newer than the one in cache.");
-       if(mode==Fetch)
-          exit(0);
-       if(mode==Real)
-          mode=Spool;
-      }
 
-    if(strlen(reply)<=256)
-       reply=(char*)realloc(reply,257);
+       if((mode==Fetch || mode==Real) && !lasttime_exists)
+          DeleteLastTimeSpoolFile(Url);
+       if(mode==Fetch)
+         {
+          if(client!=-1)
+             write_formatted(client,"Fetching %s [Page Unchanged]\n",Url->name);
+          if(spool_exists)
+             DeleteBackupWebpageSpoolFile(Url);
+          exit(0);
+         }
+       else if(mode==Real)
+         {
+          mode=Spool;
+          if(spool_exists)
+             DeleteBackupWebpageSpoolFile(Url);
+         }
+      }
+    else if(mode==Fetch && client!=-1)
+       write_formatted(client,"Fetching %s\n",Url->name);
    }
  else
-    reply=(char*)malloc(257);
+    reply_head=NULL;
+
+ reply_body=(char*)malloc(257);
 
  /* Close the outgoing file if any. */
 
@@ -545,19 +720,22 @@ int wwwoffles(int online,int browser,int client)
        CloseOutgoingSpoolFile(outgoing,Url);
    }
 
- /* Read reply and process it. */
+ /* Process the request. */
 
  if(mode==Real || mode==RealNoCache)
    {
-    int n=strlen(reply),err=0;
+    int n=strlen(reply_head),err=0;
 
-    do
+    if(mode==Real)
+       write_data(spool,reply_head,n);
+    err=write_data(client,reply_head,n);
+
+    while(err!=-1 && (n=(Url->Protocol->readbody)(reply_body,256))>0)
       {
        if(mode==Real)
-          write_data(spool,reply,n);
-       err=write_data(client,reply,n);
+          write_data(spool,reply_body,n);
+       err=write_data(client,reply_body,n);
       }
-    while(err!=-1 && (n=(Url->Protocol->read)(reply,256))>0);
 
     if(mode==Real)
        if(err==-1 || n<0)
@@ -566,14 +744,24 @@ int wwwoffles(int online,int browser,int client)
           ftruncate(spool,0);
           if(err==-1)
             {
-             RemoteHostError(spool,Url->name,"the connection was closed by the client during the transfer.");
              PrintMessage(Warning,"Error writing to client [%!s]; client disconnected?.");
+             HTMLMessage(spool,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                         "url",Url->name,
+                         "reason","ClientClose",
+                         "cache","yes",
+                         "backup",spool_exists?"yes":"",
+                         NULL);
              exit(1);
             }
           else
             {
-             RemoteHostError(spool,Url->name,"the server timed out during the transfer.");
              PrintMessage(Warning,"Timed out while reading from remote host.");
+             HTMLMessage(spool,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                         "url",Url->name,
+                         "reason","TimeoutTransfer",
+                         "cache","yes",
+                         "backup",spool_exists?"yes":"",
+                         NULL);
              exit(1);
             }
          }
@@ -582,24 +770,31 @@ int wwwoffles(int online,int browser,int client)
           int tell=lseek(spool,0,SEEK_CUR);
           if(tell!=-1)
              ftruncate(spool,tell);
+
+          if(spool_exists)
+             DeleteBackupWebpageSpoolFile(Url);
          }
    }
  else if(mode==Fetch)
    {
-    int n=strlen(reply);
+    int n=strlen(reply_head);
 
-    do
-      {
-       write_data(spool,reply,n);
-      }
-    while((n=(Url->Protocol->read)(reply,256))>0);
+    write_data(spool,reply_head,n);
+
+    while((n=(Url->Protocol->readbody)(reply_body,256))>0)
+       write_data(spool,reply_body,n);
 
     if(n<0)
       {
        lseek(spool,0,SEEK_SET);
        ftruncate(spool,0);
-       RemoteHostError(spool,Url->name,"the server timed out during the transfer.");
        PrintMessage(Warning,"Timed out while reading from remote host.");
+       HTMLMessage(spool,503,"WWWOFFLE Remote Host Error",NULL,"RemoteHostError",
+                   "url",Url->name,
+                   "reason","TimeoutTransfer",
+                   "cache","yes",
+                   "backup",spool_exists?"yes":"",
+                   NULL);
        exit(1);
       }
     else
@@ -607,78 +802,35 @@ int wwwoffles(int online,int browser,int client)
        int tell=lseek(spool,0,SEEK_CUR);
        if(tell!=-1)
           ftruncate(spool,tell);
+
+       if(spool_exists)
+          DeleteBackupWebpageSpoolFile(Url);
       }
 
-    if(FetchImages || FetchFrames)
+    if(reply_status>=200 && reply_status<400)
       {
        lseek(spool,0,SEEK_SET);
        init_buffer(spool);
 
        if(ParseHTML(spool,Url,0))
-         {
-          char **list;
-
-          if(FetchImages && (list=ListImages()))
-             for(n=0;list[n];n++)
-               {
-                char *new_request=RequestURL(list[n],Url->name);
-                int new_outgoing=OpenOutgoingSpoolFile(0);
-
-                PrintMessage(Debug,"Image=%s",list[n]);
-
-                if(new_outgoing==-1)
-                   PrintMessage(Warning,"Cannot open the new outgoing request to write.");
-                else
-                  {
-                   URL *new_Url=SplitURL(list[n]);
-                   write_string(new_outgoing,new_request);
-                   CloseOutgoingSpoolFile(new_outgoing,new_Url);
-                   FreeURL(new_Url);
-                   fetch_again++;
-                  }
-
-                free(new_request);
-               }
-
-          if(FetchFrames && (list=ListFrames()))
-             for(n=0;list[n];n++)
-               {
-                char *new_request=RequestURL(list[n],Url->name);
-                int new_outgoing=OpenOutgoingSpoolFile(0);
-
-                PrintMessage(Debug,"Frame=%s",list[n]);
-
-                if(new_outgoing==-1)
-                   PrintMessage(Warning,"Cannot open the new outgoing request to write.");
-                else
-                  {
-                   URL *new_Url=SplitURL(list[n]);
-                   write_string(new_outgoing,new_request);
-                   CloseOutgoingSpoolFile(new_outgoing,new_Url);
-                   FreeURL(new_Url);
-                   fetch_again++;
-                  }
-
-                free(new_request);
-               }
-         }
+          fetch_again=RecurseFetch(Url);
       }
    }
  else if(mode==Spool || mode==SpoolPragma)
    {
     struct stat buf;
-    char *head="HTTP/1.0 503 WWWOFFLE Remote Host Error\r\n"; /* This line must not be changed (see messages.c). */
-    int n=read_data(spool,reply,64);
+    char *head="HTTP/1.0 503 WWWOFFLE Remote Host Error\r\n";
+    int n=read_data(spool,reply_body,64);
 
-    if(!strncmp(reply,head,strlen(head)) || (!fstat(spool,&buf) && buf.st_size==0))
+    if(!strncmp(reply_body,head,strlen(head)) || (!fstat(spool,&buf) && buf.st_size==0))
+      {
        DeleteWebpageSpoolFile(Url,0);
+       RestoreBackupWebpageSpoolFile(Url);
+      }
 
-    if(AddInfoRefresh)
+    if(AddInfoRefresh && !GetHTTPHeader(request_head,"Pragma: wwwoffle"))
       {
        struct stat buf;
-       int status=0;
-
-       sscanf(reply,"%*s %d",&status);
 
        lseek(spool,0,SEEK_SET);
        init_buffer(spool);
@@ -687,104 +839,98 @@ int wwwoffles(int online,int browser,int client)
          {
           time_t t_ago=time(NULL)-buf.st_mtime;
           int position=GetHTMLEnd();
-          char *date=RFC822Date(buf.st_mtime,0),ago[24],*localhost=GetLocalHost(1);
-          char *refresh=(char*)malloc(3*strlen(Url->name)+3*strlen(localhost)+192);
-          char *index=(char*)malloc(strlen(Url->name)+strlen(localhost)+64);
-          char *msg=(char*)malloc(strlen(date)+4*strlen(Url->name)+4*strlen(localhost)+256);
+          char *date=RFC822Date(buf.st_mtime,0),*timeunit,timeago[8];
 
           if(t_ago<0)
-             sprintf(ago,"in the future");
+            {strcpy(timeago,"?");timeunit="";}
           else if(t_ago<3600)
-             sprintf(ago,"%ld min%s ago",t_ago/60,t_ago/60==1?"":"s");
+            {sprintf(timeago,"%ld",t_ago/60);timeunit=(t_ago/60==1?"min":"mins");}
           else if(t_ago<(24*3600))
-             sprintf(ago,"%ld hour%s ago",t_ago/3600,t_ago/3600==1?"":"s");
+            {sprintf(timeago,"%ld",t_ago/3600);timeunit=(t_ago/3600==1?"hour":"hours");}
           else if(t_ago<(14*24*3600))
-             sprintf(ago,"%ld day%s ago",t_ago/(24*3600),t_ago/(24*3600)==1?"":"s");
+            {sprintf(timeago,"%ld",t_ago/(24*3600));timeunit=(t_ago/(24*3600)==1?"day":"days");}
           else if(t_ago<(30*24*3600))
-             sprintf(ago,"%ld week%s ago",t_ago/(7*24*3600),t_ago/(7*24*3600)==1?"":"s");
+            {sprintf(timeago,"%ld",t_ago/(7*24*3600));timeunit=(t_ago/(7*24*3600)==1?"week":"weeks");}
           else
-             sprintf(ago,"%ld month%s ago",t_ago/(30*24*3600),t_ago/(30*24*3600)==1?"":"s");
-
-          if(Url->args && *Url->args=='!')
-             sprintf(refresh,"[<a href=\"http://%s/control/delete?url=%s\">Delete</a>"
-                             "|Refresh|"
-                             "Options]",
-                     localhost,Url->name);
-          else
-             sprintf(refresh,"[<a href=\"http://%s/control/delete?url=%s\">Delete</a>"
-                             "|<a href=\"http://%s/refresh/%s/%s\">Refresh</a>|"
-                             "<a href=\"http://%s/refresh/?%s\">Options</a>]",
-                     localhost,Url->name,
-                     localhost,Url->proto,Url->hostp,
-                     localhost,Url->name);
-
-          sprintf(index,"<a href=\"http://%s/index/%s/%s/?none\">[Index]</a>",
-                  localhost,Url->proto,Url->host);
-
-          sprintf(msg,"\n"
-                      "<hr>\n"
-                      "<p align=center>\n"
-                      "WWWOFFLE - %s (%s) - %s - %s - WWWOFFLE\n"
-                      "</p>\n"
-                      "<hr>\n",
-                  date,ago,refresh,index);
+            {sprintf(timeago,"%ld",t_ago/(30*24*3600));timeunit=(t_ago/(30*24*3600)==1?"month":"months");}
 
           lseek(spool,0,SEEK_SET);
           init_buffer(spool);
 
-          while((reply=read_line(spool,reply)))
+          while((reply_head=read_line(spool,reply_head)))
             {
-             if(!strncmp(reply,"Content-Length:",15))
-                sprintf(reply,"Content-Length: %d\r\n",atoi(&reply[16])+strlen(msg));
-             write_string(client,reply);
-             if(reply[0]=='\r' || reply[0]=='\n')
+             if(strncasecmp(reply_head,"Content-Length:",15))
+                write_string(client,reply_head);
+             if(reply_head[0]=='\r' || reply_head[0]=='\n')
                 break;
             }
-          reply=realloc(reply,257);
 
-          while(position>0 && (n=read_data(spool,reply,position>256?256:position))>0)
+          while(position>0 && (n=read_data(spool,reply_body,position>256?256:position))>0)
             {
-             write_data(client,reply,n);
+             write_data(client,reply_body,n);
              position-=n;
             }
 
-          write_string(client,msg);
+          HTMLMessageBody(client,"AddInfoRefresh",
+                          "url",Url->name,
+                          "date",date,
+                          "time",timeago,
+                          "unit",timeunit,
+                          NULL);
 
-          while((n=read_data(spool,reply,256))>0)
-             write_data(client,reply,n);
-
-          free(localhost);
-          free(refresh);
-          free(index);
-          free(msg);
+          while((n=read_data(spool,reply_body,256))>0)
+             write_data(client,reply_body,n);
          }
        else
          {
           lseek(spool,0,SEEK_SET);
           init_buffer(spool);
 
-          while((n=read_data(spool,reply,256))>0)
-             write_data(client,reply,n);
+          while((n=read_data(spool,reply_body,256))>0)
+             write_data(client,reply_body,n);
          }
       }
     else
        do
          {
-          write_data(client,reply,n);
+          write_data(client,reply_body,n);
          }
-       while((n=read_data(spool,reply,256))>0);
+       while((n=read_data(spool,reply_body,256))>0);
    }
  else if(mode==SpoolGet)
    {
-    WillGetURL(client,Url->name,0);
+    if(OfflineRequests || fetch_again)
+      {
+       if(fetch_again)
+          HTMLMessage(client,404,"WWWOFFLE Refresh Will Get",NULL,"RefreshWillGet",
+                      "url",Url->name,
+                      NULL);
+       else
+          HTMLMessage(client,404,"WWWOFFLE Will Get",NULL,"WillGet",
+                      "already","",
+                      "url",Url->name,
+                      "password",HashOutgoingSpoolFile(Url),
+                      NULL);
+      }
+    else
+       HTMLMessage(client,404,"WWWOFFLE Refused Request",NULL,"RefusedRequest",
+                   "url",Url->name,
+                   NULL);
    }
  else if(mode==SpoolWillGet)
    {
-    WillGetURL(client,Url->name,1);
+    HTMLMessage(client,404,"WWWOFFLE Will Get",NULL,"WillGet",
+                "already","yes",
+                "url",Url->name,
+                "password",HashOutgoingSpoolFile(Url),
+                NULL);
    }
  else if(mode==RealRefresh || mode==SpoolRefresh)
    {
-    RefreshRedirect(client,Url->name);
+    HTMLMessage(client,301,"WWWOFFLE Refresh Redirect",Url->link,"RefreshRedirect",
+                "url",Url->name,
+                "link",Url->link,
+                NULL);
    }
 
  /* Close down and exit. */
@@ -798,11 +944,15 @@ int wwwoffles(int online,int browser,int client)
  if(spool>=0)
     close(spool);
 
- if(request)
-    free(request);
+ if(request_head)
+    free(request_head);
+ if(request_body)
+    free(request_body);
 
- if(reply)
-    free(reply);
+ if(reply_head)
+    free(reply_head);
+ if(reply_body)
+    free(reply_body);
 
  if(client>=0)
     CloseSocket(client);
